==> alma_client-0.2.0-py3-none-any.whl: alma_client/v1/courses.py <==
import datetime
import logging
from typing import Optional, Union, List, Dict, Any

import requests

import pydantic

from .common import (
    DescribedValue,
    OptionalDescribedValue,
    parse_date,
    parse_datetime,
    AlmaModel,
    SearchWrapper,
)
from .utils import get_single_object, response2object
from .resource import AlmaResource
from ..utils import quote_path_arg

logger = logging.getLogger(__name__)


class Instructor(AlmaModel):
    primary_id: str
    first_name: str
    last_name: str


class Campus(AlmaModel):
    campus_code: DescribedValue
    campus_participants: int


class Note(AlmaModel):
    content: str
    creation_date: datetime.date
    created_by: str
    type: str  # eg "ADDRESS"

    @pydantic.validator("creation_date", pre=True)
    def parse_date(cls, value):
        return parse_date(value)


class Syllabus(AlmaModel):
    url: Optional[str]
    file: Optional[str]


class ReadingList(AlmaModel):
    link: str
    id: str
    code: str
    name: str

    status: DescribedValue
    syllabus: Syllabus
    publishingStatus: DescribedValue
    order: Optional[str]
    note: List[Note]
    description: Optional[str]
    locked: bool
    last_modified_date: datetime.datetime

    @pydantic.validator("last_modified_date", pre=True)
    def parse_date(cls, value):
        return parse_datetime(value)


class ReadingListsWrapper(AlmaModel):
    reading_list: Optional[List[ReadingList]]
    link: str


class Course(AlmaModel):
    link: str
    id: str
    code: str  # eg "UE_185_SOS2900_1_2020_VÅR_1"
    name: str

    section: str  # Typically numeric, eg "01"
    academic_department: OptionalDescribedValue
    processing_department: Optional[DescribedValue]
    term: Optional[List[DescribedValue]]
    status: str  # "ACTIVE/INACTIVE"
    start_date: Optional[datetime.date]  # Only mandatory when creating new courses
    end_date: Optional[datetime.date]  # Only mandatory when creating new courses
    weekly_hours: int
    participants: int
    year: str  # Can be an empty string, eg "2021", ""
    instructor: List[Instructor]
    campus: List[Campus]
    searchable_id: Optional[List[str]]
    note: List[Note]
    created_by: str
    created_date: datetime.date
    last_modified_by: str
    last_modified_date: datetime.date
    rolled_from: str  # Can be an empty string
    submit_by_date: Optional[datetime.date]

    @pydantic.validator(
        "start_date",
        "end_date",
        "created_date",
        "last_modified_date",
        "submit_by_date",
        pre=True,
    )
    def parse_date(cls, value):
        return parse_date(value)


class CourseSearchWrapper(SearchWrapper):
    course: Optional[List[Course]]

    def get_results(self) -> Optional[List[Course]]:
        return self.course


Data = Union[Dict[str, Any], bytes]


class Courses(AlmaResource):
    @get_single_object(Course)
    def get(self, course_id: str) -> requests.Response:
        url = "courses/{course_id}".format(course_id=quote_path_arg(course_id))
        return self.transport.get(url)

    @get_single_object(Course)
    def create(self, data: Data) -> requests.Response:
        url = "courses"
        return self.transport.post(url, data=data)

    @get_single_object(Course)
    def update(
        self,
        course_id: str,
        data: Data,
    ) -> requests.Response:
        url = "courses/{course_id}".format(course_id=quote_path_arg(course_id))
        response = self.transport.put(url, data=data)
        return response

    def list(self, limit: int = 20, offset: int = 0, q: Optional[str] = None):
        return self._list_resource(
            CourseSearchWrapper, "courses", limit=limit, offset=offset, q=q
        )

    def find(self, q: str):
        return self._find_resource(CourseSearchWrapper, "courses", q=q)

    def _list_reading_lists(self, course_id: str) -> requests.Response:
        url = "courses/{course_id}/reading-lists".format(
            course_id=quote_path_arg(course_id)
        )
        return self.transport.get(url)

    def list_reading_lists(self, course_id: str) -> List[ReadingList]:
        response: requests.Response = self._list_reading_lists(course_id)
        results = response2object(response, ReadingListsWrapper)
        if not results.reading_list:
            return []
        return results.reading_list

    @get_single_object(ReadingList)
    def create_reading_list(
        self, course_id: str, copy_id: Optional[str] = None, data: Optional[Data] = None
    ) -> requests.Response:
        url = "courses/{course_id}/reading-lists".format(
            course_id=quote_path_arg(course_id)
        )
        query = {"copy_id": copy_id} if copy_id else None
        return self.transport.post(url, query=query, data=data)

==> alma_client-0.3.0-py3-none-any.whl: alma_client/v1/courses.py <==
import datetime
import logging
from enum import Enum
from typing import Optional, Union, List, Dict, Any

import requests

import pydantic

from .common import (
    DescribedValue,
    OptionalDescribedValue,
    parse_date,
    parse_datetime,
    AlmaModel,
    SearchWrapper,
)
from .utils import get_single_object, response2object
from .resource import AlmaResource
from ..utils import quote_path_arg

logger = logging.getLogger(__name__)


class Instructor(AlmaModel):
    primary_id: str
    first_name: Optional[str]
    last_name: Optional[str]


class Campus(AlmaModel):
    campus_code: DescribedValue
    campus_participants: int


class Note(AlmaModel):
    content: str
    creation_date: datetime.date
    created_by: str
    type: Optional[str]  # eg "ADDRESS"

    @pydantic.validator("creation_date", pre=True)
    def parse_date(cls, value):
        return parse_date(value)


class Syllabus(AlmaModel):
    url: Optional[str]
    file: Optional[str]


class ReadingList(AlmaModel):
    link: str
    id: str
    code: str
    name: str

    status: DescribedValue
    syllabus: Syllabus
    publishingStatus: DescribedValue
    order: Optional[str]
    note: List[Note]
    description: Optional[str]
    locked: bool
    last_modified_date: datetime.datetime

    @pydantic.validator("last_modified_date", pre=True)
    def parse_date(cls, value):
        return parse_datetime(value)


class ReadingListsWrapper(AlmaModel):
    reading_list: Optional[List[ReadingList]]
    link: str


class Course(AlmaModel):
    link: str
    id: str
    code: str  # eg "UE_185_SOS2900_1_2020_VÅR_1"
    name: str

    section: str  # Typically numeric, eg "01"
    academic_department: OptionalDescribedValue
    processing_department: Optional[DescribedValue]
    term: Optional[List[DescribedValue]]
    status: str  # "ACTIVE/INACTIVE"
    start_date: Optional[datetime.date]  # Only mandatory when creating new courses
    end_date: Optional[datetime.date]  # Only mandatory when creating new courses
    weekly_hours: int
    participants: int
    year: str  # Can be an empty string, eg "2021", ""
    instructor: List[Instructor]
    campus: List[Campus]
    searchable_id: Optional[List[str]]
    note: List[Note]
    created_by: str
    created_date: datetime.date
    last_modified_by: str
    last_modified_date: datetime.date
    rolled_from: str  # Can be an empty string
    submit_by_date: Optional[datetime.date]

    @pydantic.validator(
        "start_date",
        "end_date",
        "created_date",
        "last_modified_date",
        "submit_by_date",
        pre=True,
    )
    def parse_date(cls, value):
        return parse_date(value)


class CourseSearchWrapper(SearchWrapper):
    course: Optional[List[Course]]

    def get_results(self) -> Optional[List[Course]]:
        return self.course


Data = Union[Dict[str, Any], bytes]


class ReadingListOwnerHierarchyValue(Enum):
    OWNER = "OWNER"
    COLLABORATOR = "COLLABORATOR"
    SUBSCRIBER = "SUBSCRIBER"


class ReadingListOwnerHierarchy(AlmaModel):
    value: Union[ReadingListOwnerHierarchyValue, str]
    desc: Optional[str]  # Owner, Collaborator

    class Config(AlmaModel.Config):
        use_enum_values = True


class ReadingListOwner(AlmaModel):
    primary_id: str
    hierarchy: ReadingListOwnerHierarchy
    full_name: str
    link: Optional[str]


class ReadingListOwnerWrapper(AlmaModel):
    owner: Optional[List[ReadingListOwner]]
    link: Optional[str]


class Courses(AlmaResource):
    @get_single_object(Course)
    def get(self, course_id: str) -> requests.Response:
        url = "courses/{course_id}".format(course_id=quote_path_arg(course_id))
        return self.transport.get(url)

    @get_single_object(Course)
    def create(self, data: Data) -> requests.Response:
        url = "courses"
        return self.transport.post(url, data=data)

    @get_single_object(Course)
    def update(
        self,
        course_id: str,
        data: Data,
    ) -> requests.Response:
        url = "courses/{course_id}".format(course_id=quote_path_arg(course_id))
        response = self.transport.put(url, data=data)
        return response

    def list(self, limit: int = 20, offset: int = 0, q: Optional[str] = None):
        return self._list_resource(
            CourseSearchWrapper, "courses", limit=limit, offset=offset, q=q
        )

    def find(self, q: str):
        return self._find_resource(CourseSearchWrapper, "courses", q=q)

    def _list_reading_lists(self, course_id: str) -> requests.Response:
        url = "courses/{course_id}/reading-lists".format(
            course_id=quote_path_arg(course_id)
        )
        return self.transport.get(url)

    def list_reading_lists(self, course_id: str) -> List[ReadingList]:
        response: requests.Response = self._list_reading_lists(course_id)
        results = response2object(response, ReadingListsWrapper)
        if not results.reading_list:
            return []
        return results.reading_list

    @get_single_object(ReadingList)
    def create_reading_list(
        self, course_id: str, copy_id: Optional[str] = None, data: Optional[Data] = None
    ) -> requests.Response:
        url = "courses/{course_id}/reading-lists".format(
            course_id=quote_path_arg(course_id)
        )
        query = {"copy_id": copy_id} if copy_id else None
        return self.transport.post(url, query=query, data=data)

    def _list_reading_list_owners(
        self,
        course_id: str,
        reading_list_id: str,
    ) -> requests.Response:
        url = "courses/{course_id}/reading-lists/{reading_list_id}/owners".format(
            course_id=quote_path_arg(course_id),
            reading_list_id=quote_path_arg(reading_list_id),
        )
        return self.transport.get(url)

    def list_reading_list_owners(
        self,
        course_id: str,
        reading_list_id: str,
    ) -> List[ReadingListOwner]:
        response = self._list_reading_list_owners(
            course_id,
            reading_list_id,
        )
        results = response2object(response, ReadingListOwnerWrapper)
        if not results.owner:
            return []
        return results.owner

    @get_single_object(ReadingListOwner)
    def create_reading_list_owner(
        self,
        course_id: str,
        reading_list_id: str,
        owner_primary_id: str,
    ):
        url = "courses/{course_id}/reading-lists/{reading_list_id}/owners".format(
            course_id=quote_path_arg(course_id),
            reading_list_id=quote_path_arg(reading_list_id),
        )
        owner_data = {
            "primary_id": owner_primary_id,
            "hierarchy": {
                "value": ReadingListOwnerHierarchyValue.OWNER.value,
            },
        }
        return self.transport.post(
            path=url,
            data=owner_data,
        )

    def delete_reading_list_owner(
        self,
        course_id: str,
        reading_list_id: str,
        owner_primary_id: str,
    ) -> bool:
        if not owner_primary_id:
            return False

        url = "courses/{course_id}/reading-lists/{reading_list_id}/owners/{primary_id}".format(
            course_id=quote_path_arg(course_id),
            reading_list_id=quote_path_arg(reading_list_id),
            primary_id=quote_path_arg(owner_primary_id),
        )
        response = self.transport.delete(path=url)
        return response.status_code == 204
